# -*- coding: utf-8 -*-
from __future__ import (absolute_import, print_function, unicode_literals)
from acli.output import output_ascii_table, output_ascii_table_list, dash_if_none
from colorclass import Color, Windows
from external.six import iteritems

Windows.enable(auto_colors=True, reset_atexit=True)


def get_elb_instances(instances=None):
    """
    @type instances: list
    """
    instance_ids = [x.id for x in instances]
    return ",".join(instance_ids)


def output_elbs(elbs=None):
    """
    @type elbs: list
    """
    if elbs:
        elbs.sort(key=lambda k: k.get('DNSName'))
        td = list()
        table_header = [Color('{autoblue}name{/autoblue}'),
                        Color('{autoblue}instances{/autoblue}'),
                        Color('{autoblue}dns name{/autoblue}')]
        for elb in elbs:
            td.append([elb.get('LoadBalancerName'), str(len(elb.get('Instances'))), elb.get('DNSName')])
        output_ascii_table_list(table_title=Color('{autowhite}ELBs{/autowhite}'),
                                table_data=td,
                                table_header=table_header,
                                inner_heading_row_border=True)
    else:
        print("No ELBs found.")
    exit(0)


def get_elb_policies(policies=None):
    """
    @type policies: Policies
    """
    output = ""
    if policies.app_cookie_stickiness_policies:
        for acsp in policies.app_cookie_stickiness_policies:
            output += "{0}\n".format(str(acsp))
    if policies.lb_cookie_stickiness_policies:
        for lcsp in policies.lb_cookie_stickiness_policies:
            output += "{0}\n".format(str(lcsp))
    if policies.other_policies:
        for op in policies.lb_cookie_stickiness_policies:
            output += "{0}\n".format(str(op))
    if output:
        return output.rstrip()


def get_elb_listeners(listeners=None):
    """
    @type listeners: list
    """
    output = ""
    for listener in listeners:
        listener_config = listener.get('Listener')
        output += "LB Port: {0} \n" \
                  "LB Protocol: {1} \n" \
                  "Instance Port: {2} \n" \
                  "Instance Protocol: " \
                  "{3}\n".format(listener_config.get('LoadBalancerPort', '-'),
                                 listener_config.get('Protocol', '-'),
                                 listener_config.get('InstancePort', '-'),
                                 listener_config.get('InstanceProtocol', '-'))
    if output:
        return output.rstrip()


def get_source_secgroup_name(source_secgroup=None):
    if source_secgroup:
        return source_secgroup.get('GroupName')


def get_healthcheck(hc=None):
    if hc:
        out = str()
        for key, value in iteritems(hc):
            out += "{0}: {1}\n".format(key, value)
        return out.rstrip()


def output_elb_info(elb=None):
    """
    @type elb: dict
    """
    td = list()
    td.append([Color('{autoblue}name{/autoblue}'),
               elb.get('LoadBalancerName')])
    td.append([Color('{autoblue}dns name{/autoblue}'),
               elb.get('DNSName')])
    # print(elb.get('ListenerDescriptions'))
    td.append([Color('{autoblue}listeners{/autoblue}'),
               dash_if_none(get_elb_listeners(elb.get('ListenerDescriptions')))])
    td.append([Color('{autoblue}canonical hosted zone name{/autoblue}'),
               dash_if_none(elb.get('CanonicalHostedZoneName'))])
    td.append([Color('{autoblue}canonical hosted zone name id{/autoblue}'),
               dash_if_none(elb.get('CanonicalHostedZoneNameID'))])
    # td.append(['connection', str(elb.connection)])
    # td.append(['policies', dash_if_none(get_elb_policies(elb.get('Policies)))])
    td.append([Color('{autoblue}health check{/autoblue}'),
               dash_if_none(get_healthcheck(elb.get('HealthCheck')))])
    td.append([Color('{autoblue}created{/autoblue}'),
               str(dash_if_none(elb.get('CreatedTime').replace(tzinfo=None, microsecond=0)))])
    # td.append(['instances', get_elb_instances(elb.get('Instances'))])
    td.append([Color('{autoblue}availability zones{/autoblue}'),
               ",".join(elb.get('AvailabilityZones'))])
    td.append([Color('{autoblue}source security group{/autoblue}'),
               dash_if_none(get_source_secgroup_name(elb.get('SourceSecurityGroup')))])
    td.append([Color('{autoblue}security groups{/autoblue}'),
               ",".join(elb.get('SecurityGroups'))])
    td.append([Color('{autoblue}subnets{/autoblue}'),
               ",".join(elb.get('Subnets'))])
    td.append([Color('{autoblue}vpc id{/autoblue}'),
               dash_if_none(elb.get('VPCId'))])
    output_ascii_table(table_title=Color('{autowhite}elb info{/autowhite}'),
                       table_data=td)
    exit(0)



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, print_function, unicode_literals)

from colorclass import Color, Windows

from acli.output import (output_ascii_table, output_ascii_table_list)

Windows.enable(auto_colors=True, reset_atexit=True)
from external.six import iteritems


def get_tag(name=None, tags=None):
    if tags:
        for tag in tags:
            if tag.get('Key') == name:
                return tag.get('Value')


def output_domain_list(domains=None):
    """
    @type domains: dict
    """
    td = list()
    table_header = [Color('{autoblue}domain name{/autoblue}')]
    for domain in domains:
        td.append([domain.get('DomainName')])
    output_ascii_table_list(table_title=Color('{autowhite}ES domains{/autowhite}'),
                            table_data=td,
                            table_header=table_header,
                            inner_heading_row_border=True)
    exit(0)


def output_dict(dict_=None):
    """
    @type dict_: dict
    """
    output = list()
    for k, v in iteritems(dict_):
        output.append('{0}: {1}\n'.format(k, str(v)))
    return ''.join(output).rstrip()


def colour_created(state=None):
    if not state:
        return Color('{autoyellow}False{/autoyellow}')
    else:
        return Color('{autogreen}True{/autogreen}')


def colour_deleted(state=None):
    if not state:
        return Color('{autogreen}False{/autogreen}')
    else:
        return Color('{autored}True{/autored}')


def colour_processing(state=None):
    if not state:
        return Color('{autogreen}False{/autogreen}')
    else:
        return Color('{autoyellow}True{/autoyellow}')


def output_domain_info(domain=None):
    """
    @type domain: dict
    """
    if domain:
        domain_details = domain.get('DomainStatusList')[0]
        cluster_conf = domain_details.get('ElasticsearchClusterConfig')
        td = list()
        td.append([Color('{autoblue}domain name{/autoblue}'),
                   domain_details.get('DomainName')])
        td.append([Color('{autoblue}endpoint{/autoblue}'),
                   domain_details.get('Endpoint')])
        td.append([Color('{autoblue}created{/autoblue}'),
                   colour_created(domain_details.get('Created'))])
        td.append([Color('{autoblue}deleted{/autoblue}'),
                   colour_deleted(domain_details.get('Deleted'))])
        td.append([Color('{autoblue}processing{/autoblue}'),
                   colour_processing(domain_details.get('Processing'))])
        td.append([Color('{autoblue}cluster config{/autoblue}'),
                   ' '])
        td.append([Color('{autoblue} dedicated master enabled{/autoblue}'),
                   str(cluster_conf.get('DedicatedMasterEnabled'))])
        td.append([Color('{autoblue} instance type{/autoblue}'),
                   str(cluster_conf.get('InstanceType'))])
        td.append([Color('{autoblue} instance count{/autoblue}'),
                   str(cluster_conf.get('InstanceCount'))])
        td.append([Color('{autoblue} zone awareness{/autoblue}'),
                   str(cluster_conf.get('ZoneAwarenessEnabled'))])
        td.append([Color('{autoblue}domain id{/autoblue}'),
                   domain_details.get('DomainId')])
        td.append([Color('{autoblue}snapshot options{/autoblue}'),
                   output_dict(domain_details.get('SnapshotOptions'))])
        td.append([Color('{autoblue}advanced options{/autoblue}'),
                   output_dict(domain_details.get('AdvancedOptions'))])
        td.append([Color('{autoblue}ARN{/autoblue}'),
                   domain_details.get('ARN')])
        output_ascii_table(table_title=Color('{autowhite}ES domain info{/autowhite}'),
                           table_data=td)
    else:
        exit('Domain does not exist.')
    exit(0)



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, print_function, unicode_literals)
from acli.output import output_ascii_table, output_ascii_table_list, dash_if_none, get_tags
from colorclass import Color, Windows
Windows.enable(auto_colors=True, reset_atexit=True)


def get_instances_output(instances):
    """
    @type instances: list
    """
    ret = ""
    for instance in instances:
        ret += "{0}\n".format(instance.get('InstanceId'))
    if ret:
        return ret.rstrip()


def output_asg_list(asg_list=None):
    """
    @type asg_list: list
    """
    if isinstance(asg_list, list):
        td = list()
        table_header = [Color('{autoblue}name{/autoblue}'), Color('{autoblue}instances{/autoblue}'),
                        Color('{autoblue}desired{/autoblue}'), Color('{autoblue}min{/autoblue}'),
                        Color('{autoblue}max{/autoblue}'),
                        Color('{autoblue}lc{/autoblue}')]
        for asg in asg_list:
            td.append([asg.get('AutoScalingGroupName', '-'),
                       str(len(asg.get('Instances', '-'))),
                       str(asg.get('DesiredCapacity', '-')),
                       str(asg.get('MinSize', '-')),
                       str(asg.get('MaxSize', '-')),
                       asg.get('LaunchConfigurationName', '-')
                       ])
        output_ascii_table_list(table_title=Color('{autowhite}ASGs{/autowhite}'),
                                table_data=td,
                                table_header=table_header,
                                inner_heading_row_border=True)
    exit(0)


def output_asg_info(asg=None):
    """
    @type asg: dict
    """
    td = list()
    td.append([Color('{autoblue}name{/autoblue}'),
               dash_if_none(asg.get('AutoScalingGroupName'))])
    # td.append([Color('{autoblue}arn{/autoblue}'), dash_if_none(asg.get('AutoScalingGroupARN'))])
    td.append([Color('{autoblue}lc{/autoblue}'),
               dash_if_none(asg.get('LaunchConfigurationName'))])
    td.append([Color('{autoblue}min size{/autoblue}'),
               str(dash_if_none(asg.get('MinSize')))])
    td.append([Color('{autoblue}max size{/autoblue}'),
               str(dash_if_none(asg.get('MaxSize')))])
    td.append([Color('{autoblue}desired size{/autoblue}'),
               str(dash_if_none(asg.get('DesiredCapacity')))])
    td.append([Color('{autoblue}default cooldown{/autoblue}'),
               str(dash_if_none(asg.get('DefaultCooldown')))])
    td.append([Color('{autoblue}availability zones{/autoblue}'),
               str(dash_if_none('\n'.join(asg.get('AvailabilityZones'))))])
    td.append([Color('{autoblue}load balancer names{/autoblue}'),
               str(dash_if_none('\n'.join(asg.get('LoadBalancerNames'))))])
    td.append([Color('{autoblue}health check type{/autoblue}'),
               str(dash_if_none(asg.get('HealthCheckType')))])
    td.append([Color('{autoblue}health check grace period{/autoblue}'),
               str(dash_if_none(asg.get('HealthCheckGracePeriod')))])
    td.append([Color('{autoblue}instances{/autoblue}'),
               dash_if_none(get_instances_output(asg.get('Instances')))])
    td.append([Color('{autoblue}created{/autoblue}'),
               str(dash_if_none(asg.get('CreatedTime').replace(tzinfo=None, microsecond=0)))])
    td.append([Color('{autoblue}suspended processes{/autoblue}'),
               dash_if_none(asg.get('SuspendedProcesses'))])
    td.append([Color('{autoblue}placement group{/autoblue}'),
               dash_if_none(asg.get('PlacementGroup'))])
    td.append([Color('{autoblue}vpc zone identifier{/autoblue}'),
               str(dash_if_none(asg.get('VPCZoneIdentifier')))])
    td.append([Color('{autoblue}metrics enabled{/autoblue}'),
               dash_if_none(asg.get('EnabledMetrics'))])
    td.append([Color('{autoblue}status{/autoblue}'),
               dash_if_none(asg.get('Status'))])
    td.append([Color('{autoblue}tags{/autoblue}'),
               dash_if_none(get_tags(asg.get('Tags')))])
    td.append([Color('{autoblue}termination policies{/autoblue}'),
               str(dash_if_none(asg.get('TerminationPolicies')))])
    output_ascii_table(table_title=Color('{autowhite}asg info{/autowhite}'),
                       table_data=td)
    exit(0)


def output_lc_list(lc_list=None):
    """
    @type lc_list: list
    """
    if isinstance(lc_list, list):
        td = list()
        table_header = [Color('{autoblue}name{/autoblue}'),
                        Color('{autoblue}image id{/autoblue}'),
                        Color('{autoblue}instance type{/autoblue}'),
                        Color('{autoblue}created{/autoblue}')]
        for lc in lc_list:
            td.append([
                      dash_if_none(lc.get('LaunchConfigurationName')),
                      dash_if_none(lc.get('ImageId')),
                      dash_if_none(lc.get('InstanceType')),
                      dash_if_none(lc.get('CreatedTime').replace(tzinfo=None, microsecond=0))
                      ])
        output_ascii_table_list(table_title=Color('{autowhite}launch configurations{/autowhite}'),
                                table_header=table_header,
                                table_data=td,
                                inner_heading_row_border=True)
    exit(0)


def output_lc_info(lc=None):
    """
    @type lc: dict
    """
    td = list()
    td.append([Color('{autoblue}name{/autoblue}'),
               dash_if_none(lc.get('LaunchConfigurationName'))])
    td.append([Color('{autoblue}image id{/autoblue}'),
               dash_if_none(lc.get('ImageId'))])
    td.append([Color('{autoblue}instance type{/autoblue}'),
               dash_if_none(lc.get('InstanceType'))])
    td.append([Color('{autoblue}created (UTC){/autoblue}'),
               str(dash_if_none(lc.get('CreatedTime').replace(tzinfo=None, microsecond=0)))])
    td.append([Color('{autoblue}iam instance profile{/autoblue}'),
               dash_if_none(lc.get('IamInstanceProfile'))])
    td.append([Color('{autoblue}ebs optimised{/autoblue}'),
               dash_if_none("Yes" if lc.get('EbsOptimized') else "No")])
    td.append([Color('{autoblue}instance monitoring{/autoblue}'),
               dash_if_none("Yes" if lc.get('InstanceMonitoring').get('Enabled') else "No")])
    td.append([Color('{autoblue}classiclink VPC sec groups{/autoblue}'),
               dash_if_none(lc.get('ClassicLinkVPCSecurityGroups'))])
    td.append([Color('{autoblue}block device mappings{/autoblue}'),
               dash_if_none(lc.get('BlockDeviceMappings'))])
    td.append([Color('{autoblue}keypair{/autoblue}'),
               dash_if_none(lc.get('KeyName'))])
    td.append([Color('{autoblue}security groups{/autoblue}'),
               dash_if_none(", ".join(lc.get('SecurityGroups')))])
    td.append([Color('{autoblue}kernel ID{/autoblue}'),
               dash_if_none(lc.get('KernelId'))])
    td.append([Color('{autoblue}ramdisk id{/autoblue}'),
               dash_if_none(lc.get('RamdiskId'))])
    td.append([Color('{autoblue}image id{/autoblue}'),
               dash_if_none(lc.get('ImageId'))])
    td.append([Color('{autoblue}instance typr{/autoblue}'),
               dash_if_none(lc.get('InstanceType'))])
    td.append([Color('{autoblue}associate public ip{/autoblue}'),
               dash_if_none(lc.get('AssociatePublicIpAddress'))])
    td.append([Color('{autoblue}user data{/autoblue}'),
               dash_if_none(lc.get('UserData'))])
    output_ascii_table(table_title=Color('{autowhite}launch configuration info{/autowhite}'),
                       table_data=td)
    exit(0)



# -*- coding: utf-8 -*-
"""Usage:
    acli ami (ls | list) [options]
    acli ami info <ami_id>

    -f, --filter=<term>             filter results by term
    -h, --help
"""
from __future__ import (absolute_import, print_function, unicode_literals)
from docopt import docopt

if __name__ == '__main__':
    print(docopt(__doc__))



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, print_function, unicode_literals)

from botocore.exceptions import NoCredentialsError, NoRegionError, ClientError


def handle_boto_errors(function):
    def handle_errors(*args, **kwargs):
        try:
            return function(*args, **kwargs)
        except NoCredentialsError:
            exit('Credentials not found. See here for more information:\n'
                 'http://boto3.readthedocs.org/en/latest/guide/configuration.html#configuration-files')
        except NoRegionError:
            exit('Cannot perform this task without specifying an AWS region.\n'
                 'Please check your boto/aws settings or specify using \'acli --region=<region>\'.')
        except ClientError as ce:
            response = ce.response
            error_args = ce.args
            error_message = response['Error']['Message']
            error_code = response['Error']['Code']
            joined_args = "".join(error_args)
            if error_code in ('AccessDenied', 'AccessDeniedException'):
                if 'iam:ListUsers' in error_message:
                    exit('You do not have permission to access IAM user list.\nDetail: {0}'.format(error_message))
                elif 'DescribeLaunchConfigurations' in joined_args:
                    exit('You do not have permission to list Launch Configurations.')
                elif 'DescribeAutoScalingGroups' in joined_args:
                    exit('You do not have permission to list Auto Scaling Groups.')
                elif 'DescribeLoadBalancers' in joined_args:
                    exit('You do not have permission to list Elastic Load Balancers.')
                elif 'ListDomainNames' in joined_args:
                    exit('You do not have permission to list Elastic Search domains.')
                elif 'ListHostedZones' in joined_args:
                    exit('You do not have permission to list Route53 zones.')
                else:
                    raise
            elif error_code == 'UnauthorizedOperation':
                if 'DescribeImages' in joined_args:
                    exit('You do not have permission to list AMIs.')
                elif 'DescribeInstances' in joined_args:
                    exit('You do not have permission to list instances.')
                elif 'DescribeAddresses' in joined_args:
                    exit('You do not have permission to list addresses.')
                elif 'DescribeVpcs' in joined_args:
                    exit('You do not have permission to list VPCs.')
                elif 'DescribeSecurityGroups' in joined_args:
                    exit('You do not have permission to list Security Groups.')

                else:
                    raise
            elif error_code == "RequestExpired":
                exit("Temporary credentials have expired.")
            else:
                exit(ce)
        except Exception:
            raise
    return handle_errors



from __future__ import (absolute_import, print_function, unicode_literals)
from acli.services.asg import (asg_list, asg_info, lc_list, lc_info)
from acli.config import Config
from moto import mock_ec2, mock_autoscaling

import pytest
from boto3.session import Session
session = Session(region_name="eu-west-1")


@pytest.yield_fixture(scope='function')
def fake_lc():
    """ASG mock service"""
    mock = mock_autoscaling()
    mock.start()
    client = session.client('autoscaling')
    client.create_launch_configuration(
        LaunchConfigurationName='test_lc',
        ImageId='ami-abcd1234',
        InstanceType='t2.medium'
    )
    yield client.describe_launch_configurations()
    mock.stop()


#@pytest.yield_fixture(scope='function')
#def test_create_autoscaling_group():
#    mock = mock_autoscaling()
#    mock.start()
#    conn = session.client('autoscaling')
#    conn.create_launch_configuration(LaunchConfigurationName='test_lc',
#                                     ImageId='ami-abcd1234',
#                                     InstanceType='t2.medium')
#    yield conn.describe_launch_configurations()
#    mock.stop()


@pytest.yield_fixture(scope='function')
def fake_asg():
    """ASG mock service"""
    mock = mock_autoscaling()
    mock.start()
    client = session.client('autoscaling')
    client.create_launch_configuration(
        LaunchConfigurationName='test_lc',
        ImageId='ami-abcd1234',
        InstanceType='t2.medium'
    )

    client.create_auto_scaling_group(
        AutoScalingGroupName='test_asg',
        LaunchConfigurationName='test_lc',
        InstanceId='string',
        MinSize=123,
        MaxSize=123,
        DesiredCapacity=123,
        DefaultCooldown=123,
        AvailabilityZones=[
            'string',
        ],
        # LoadBalancerNames=[
        #    'string',
        # ],
        HealthCheckType='string',
        HealthCheckGracePeriod=123,
        PlacementGroup='string',
        VPCZoneIdentifier='string',
        TerminationPolicies=[
            'string',
        ],
        Tags=[
            {
                'ResourceId': 'string',
                'ResourceType': 'string',
                'Key': 'string',
                'Value': 'string',
                'PropagateAtLaunch': True
            },
        ]
    )
    yield client.describe_auto_scaling_groups()
    mock.stop()


config = Config(cli_args={'--region': 'eu-west-1',
                          '--access_key_id': 'AKIAIOSFODNN7EXAMPLE',
                          '--secret_access_key': 'wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMPLEKEY'})


def test_lc_list_service(fake_lc):
    with pytest.raises(SystemExit):
        assert lc_list(aws_config=config)


def test_lc_info_service(fake_lc):
    with pytest.raises(SystemExit):
        assert lc_info(aws_config=config, lc_name='test_lc')


def test_asg_list_service(fake_asg):
    with pytest.raises(SystemExit):
        assert lc_list(aws_config=config)


def test_asg_info_asgs(fake_asg):
    with pytest.raises(SystemExit):
        assert asg_info(aws_config=config, asg_name='test_asg')



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, print_function, unicode_literals)

import humanize
from colorclass import Color, Windows

from acli.output import (output_ascii_table, output_ascii_table_list, dash_if_none)
from external.six import iteritems

Windows.enable(auto_colors=True, reset_atexit=True)


def get_tag(name=None, tags=None):
    if tags:
        for tag in tags:
            if tag.get('Key') == name:
                return tag.get('Value')


def colour_state(state=None):
    if not state:
        return Color('{autoblack}-{/autoblack}')
    elif state == 'available':
        return Color('{autogreen}' + state + '{/autogreen}')
    elif state in ('deleting', 'deleted'):
        return Color('{autored}' + state + '{/autored}')
    elif state == 'creating':
        return Color('{autoyellow}' + state + '{/autoyellow}')


def output_filesystems(filesystems=None, mount_targets=None):
    """
    @type filesystems: dict
    @type mount_targets: list
    """
    td = list()
    table_header = [Color('{autoblue}id{/autoblue}'),
                    Color('{autoblue}name{/autoblue}'),
                    Color('{autoblue}state{/autoblue}'),
                    Color('{autoblue}size / time (UTC){/autoblue}'),
                    Color('{autoblue}mode{/autoblue}'),
                    Color('{autoblue}mount targets{/autoblue}')
                    ]
    fs_ids = list()
    for fs in filesystems:
        size_in_bytes = fs.get('SizeInBytes')
        size_in_bytes_value = size_in_bytes.get('Value')
        if size_in_bytes_value:
            size_in_bytes_value = humanize.naturalsize(size_in_bytes_value)
        size_in_bytes_timestamp = size_in_bytes.get('Timestamp')
        if size_in_bytes_timestamp:
            size_in_bytes_timestamp = size_in_bytes_timestamp.replace(tzinfo=None, second=0)
        created_time = fs.get('CreationTime')
        if created_time:
            created_time = created_time.replace(tzinfo=None, second=0)

        td.append([fs.get('FileSystemId'),
                   dash_if_none(fs.get('Name')),
                   colour_state(fs.get('LifeCycleState')),
                   '{0} / {1}'.format(size_in_bytes_value,
                                      dash_if_none(size_in_bytes_timestamp)),
                   fs.get('PerformanceMode'),
                   dash_if_none(fs.get('NumberOfMountTargets'))
                   ])
        fs_ids.append(fs.get('FileSystemId'))
    output_ascii_table_list(table_title=Color('{autowhite}EFS Filesystems{/autowhite}'),
                            table_data=td,
                            table_header=table_header,
                            inner_heading_row_border=True)
    # Output mount targets
    td = list()
    table_header = [Color('{autoblue}mount target id{/autoblue}'),
                    Color('{autoblue}filesystem id{/autoblue}'),
                    Color('{autoblue}lifecycle state{/autoblue}')
                    ]
    for mt in mount_targets:
        td.append([mt.get('MountTargetId'),
                   mt.get('FileSystemId'),
                   colour_state(mt.get('LifeCycleState'))
                   ])
    output_ascii_table_list(table_title=Color('{autowhite}EFS Mount Targets{/autowhite}'),
                            table_data=td,
                            table_header=table_header,
                            inner_heading_row_border=True)
    exit(0)


def output_dict(dict_=None):
    """
    @type dict_: dict
    """
    output = list()
    for k, v in iteritems(dict_):
        output.append('{0}: {1}\n'.format(k, str(v)))
    return ''.join(output).rstrip()


def colour_created(state=None):
    if not state:
        return Color('{autoyellow}False{/autoyellow}')
    else:
        return Color('{autogreen}True{/autogreen}')


def colour_deleted(state=None):
    if not state:
        return Color('{autogreen}False{/autogreen}')
    else:
        return Color('{autored}True{/autored}')


def colour_processing(state=None):
    if not state:
        return Color('{autogreen}False{/autogreen}')
    else:
        return Color('{autoyellow}True{/autoyellow}')


def output_filesystem_info(filesystem=None, mount_targets=None):
    """
    @type filesystem: dict
    @type mount_targets: list
    """
    if filesystem:
        filesystem_details = filesystem.get('FileSystems')[0]
        td = list()
        td.append([Color('{autoblue}filesystem name{/autoblue}'),
                   dash_if_none(filesystem_details.get('Name'))])
        td.append([Color('{autoblue}id{/autoblue}'),
                   filesystem_details.get('FileSystemId')])
        td.append([Color('{autoblue}size{/autoblue}'),
                   filesystem_details['SizeInBytes']['Value']])
        td.append([Color('{autoblue}owner id{/autoblue}'),
                   dash_if_none(filesystem_details.get('OwnerId'))])
        td.append([Color('{autoblue}creation time{/autoblue}'),
                   dash_if_none(filesystem_details.get('CreationTime'))])
        td.append([Color('{autoblue}lifecycle state{/autoblue}'),
                   dash_if_none(filesystem_details.get('LifeCycleState'))])
        td.append([Color('{autoblue}no. mount targets{/autoblue}'),
                   dash_if_none(filesystem_details.get('NumberOfMountTargets'))])
        td.append([Color('{autoblue}performance mode{/autoblue}'),
                   dash_if_none(filesystem_details.get('PerformanceMode'))])
        td.append([Color('{autoblue}creation token{/autoblue}'),
                   dash_if_none(filesystem_details.get('CreationToken'))])
        output_ascii_table(table_title=Color('{autowhite}EFS filesystem info{/autowhite}'),
                           table_data=td)
    else:
        exit('filesystem does not exist.')

    if mount_targets:
        table_header = [Color('{autoblue}mount target id{/autoblue}'),
                        Color('{autoblue}filesystem id{/autoblue}'),
                        Color('{autoblue}lifecycle state{/autoblue}'),
                        Color('{autoblue}ip address{/autoblue}'),
                        Color('{autoblue}subnet id{/autoblue}'),
                        Color('{autoblue}interface id{/autoblue}'),
                        Color('{autoblue}owner id{/autoblue}')
                        ]
        td = list()
        for mt in mount_targets:
            td.append([mt.get('MountTargetId'),
                       mt.get('FileSystemId'),
                       colour_state(mt.get('LifeCycleState')),
                       mt.get('IpAddress'),
                       mt.get('SubnetId'),
                       mt.get('NetworkInterfaceId'),
                       mt.get('OwnerId')
                       ])
        output_ascii_table_list(table_title=Color('{autowhite}EFS Mount Targets{/autowhite}'),
                                table_header=table_header,
                                table_data=td,
                                inner_heading_row_border=True)
    exit(0)



# -*- coding: utf-8 -*-
"""Usage:
    acli iam summary
    acli iam user (ls | list) [options]
    acli iam user info <username>

    -o, --output=<output_type>      table, json, yaml or graph [default: table].
    -h, --help
"""
from __future__ import (absolute_import, print_function, unicode_literals)
from docopt import docopt

if __name__ == '__main__':
    print(docopt(__doc__))



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, print_function, unicode_literals)

import math
import os

from colorclass import Color, Windows
from terminaltables import AsciiTable

from acli.utils import get_console_dimensions

try:
    input = raw_input
except NameError:
    pass
Windows.enable(auto_colors=True, reset_atexit=True)


def output_ascii_table(table_title=None,
                       table_data=None,
                       inner_heading_row_border=False,
                       inner_footing_row_border=False,
                       inner_row_border=False):
    """
    @type table_title: unicode
    @type table_data: list
    @type inner_heading_row_border: bool
    @type inner_footing_row_border: bool
    @type inner_row_border: bool
    """
    table = AsciiTable(table_data)
    table.inner_heading_row_border = inner_heading_row_border
    table.inner_row_border = inner_row_border
    table.inner_footing_row_border = inner_footing_row_border
    table.title = table_title
    print(table.table)


def output_ascii_table_list(table_title=None,
                            table_data=None,
                            table_header=None,
                            inner_heading_row_border=False,
                            inner_row_border=False):
    """
    @type table_title: unicode
    @type table_data: list
    @type inner_heading_row_border: bool
    @type inner_row_border: bool
    @type table_header: list
    """
    console_rows, _ = get_console_dimensions()
    console_rows = int(console_rows)
    full_display_length = len(table_data) + 7
    items_per_page = console_rows - 7
    num_pages = 0
    if full_display_length > console_rows:
        try:
            num_pages = int(math.ceil(float(len(table_data)) / float(items_per_page)))
        except ZeroDivisionError:
            exit('Console too small to display.')
    if num_pages:
        running_count = 0
        for page in range(1, num_pages + 1):
            page_table_output = list()
            page_table_output.insert(0, table_header)
            upper = (console_rows + running_count) - 7
            if upper > len(table_data):
                upper = len(table_data)
            for x in range(running_count, upper):
                page_table_output.append(table_data[x])
                running_count += 1
            table = AsciiTable(page_table_output)
            table.inner_heading_row_border = inner_heading_row_border
            table.inner_row_border = inner_row_border
            table.title = table_title
            if page != 1:
                print('')
            print(table.table)
            if page < num_pages:
                input("Press Enter to continue...")
                os.system('clear')
    else:
        table_data.insert(0, table_header)
        table = AsciiTable(table_data)
        table.inner_heading_row_border = inner_heading_row_border
        table.inner_row_border = inner_row_border
        table.title = table_title
        print(table.table)


def dash_if_none(item=None):
    """
    @type item: object
    """
    return str(item) if item else Color('{autoblack}-{/autoblack}')


def get_tags(tags, separator=', '):
    """
    @type tags: list
    @type separator: unicode
    """
    tag_list = list()
    for tag in tags:
        tag_list.append("{0}:{1}".format(tag.get('Key'), tag.get('Value')))
    if tag_list:
        return separator.join(tag_list)


def get_name_tag(tags):
    """
    @type tags: dict
    """
    for tag_name, tag_value in tags.iteritems():
        if tag_name == 'Name':
            return tag_value
    return "-"



# -*- coding: utf-8 -*-
"""Usage:
    acli ec2 (ls | list | summary) [options] [--region=<region>]
    acli ec2 (start | stop | reboot | terminate | info | cpu | vols | net) <instance_id> [options]

    -f, --filter=<term>             filter results by term
    -s, --start=<start_date>        metrics start-date
    -e, --end=<end_date>            metrics end-date
    -p, --period=<period>           metrics period
    -i, --intervals=<intervals>     metrics intervals
    -h, --help
"""
from __future__ import (absolute_import, print_function, unicode_literals)
from docopt import docopt
from acli.services import (ec2, cloudwatch)


def ec2_command(argv=None, aws_config=None):
    ec2_res = docopt(__doc__, argv=argv)
    if any((ec2_res.get('ls'), ec2_res.get('list'))):
        ec2.ec2_list(aws_config, filter_term=ec2_res.get('--filter'))
    elif ec2_res.get('info'):
        ec2.ec2_info(aws_config, instance_id=ec2_res.get('<instance_id>'))
    elif ec2_res.get('stop'):
        ec2.ec2_manage(aws_config, instance_id=ec2_res.get('<instance_id>'), action="stop")
    elif ec2_res.get('reboot'):
        ec2.ec2_manage(aws_config, instance_id=ec2_res.get('<instance_id>'), action="reboot")
    elif ec2_res.get('start'):
        ec2.ec2_manage(aws_config, instance_id=ec2_res.get('<instance_id>'), action="start")
    elif ec2_res.get('terminate'):
        ec2.ec2_manage(aws_config, instance_id=ec2_res.get('<instance_id>'), action="terminate")
    elif ec2_res.get('cpu'):
        cloudwatch.ec2_cpu(aws_config=aws_config, instance_id=ec2_res.get('<instance_id>'))
    elif ec2_res.get('net'):
        cloudwatch.ec2_net(aws_config=aws_config,
                           instance_id=ec2_res.get('<instance_id>'),
                           start=ec2_res.get('--start'),
                           period=ec2_res.get('--end'),
                           intervals=ec2_res.get('intervals')
                           )
    elif ec2_res.get('vols'):
        cloudwatch.ec2_vol(aws_config=aws_config,
                           instance_id=ec2_res.get('<instance_id>'),
                           start=ec2_res.get('--start'),
                           period=ec2_res.get('--end'),
                           intervals=ec2_res.get('intervals')
                           )
    elif ec2_res.get('summary'):
        ec2.ec2_summary(aws_config=aws_config)

if __name__ == '__main__':
    print(docopt(__doc__))



# -*- coding: utf-8 -*-
"""Usage:
    acli elb (ls | list)
    acli elb info <elb_name>

    -h, --help
"""
from __future__ import (absolute_import, print_function, unicode_literals)
from docopt import docopt

if __name__ == '__main__':
    print(docopt(__doc__))



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, print_function, unicode_literals)

import hashlib
import io

from botocore.exceptions import ClientError

from acli.connections import get_client
from acli.errors import handle_boto_errors
from acli.output.s3 import (output_s3_list, output_s3_info)


@handle_boto_errors
def check_bucket_accessible(s3_client=None, bucket_name=None):
    try:
        s3_client.head_bucket(Bucket=bucket_name)
        return True
    except ClientError:
        exit("Unable to access bucket.")
    except Exception as unhandled:
        exit('Unhandled exception: {0}'.format(unhandled))


@handle_boto_errors
def get_s3_file_md5(s3_client=None, bucket_name=None, path=None):
    try:
        key_detail = s3_client.head_object(Bucket=bucket_name, Key=path)
        return key_detail.get('ETag')[1:-1]
    except ClientError:
        pass


def get_local_file_md5(path=None):
    try:
        with io.open(path, "rb") as local_file:
            return hashlib.md5(local_file.read()).hexdigest()
    except IOError:
        pass


@handle_boto_errors
def s3_list(aws_config=None, item=None):
    """
    @type aws_config: Config
    @type item: unicode
    """
    s3_client = get_client(client_type='s3', config=aws_config)
    buckets = s3_client.list_buckets()
    if not item:
        if buckets.get('Buckets'):
            output_s3_list(buckets=buckets.get('Buckets'))
        else:
            exit("No buckets found.")
    else:
        prefix = ''
        if item and '/' in item:
            path_elements = item.split('/')
            bucket_name = path_elements[0]
            prefix = "/".join(path_elements[1:])
        else:
            bucket_name = item
        check_bucket_accessible(s3_client=s3_client, bucket_name=bucket_name)
        try:
            objects = s3_client.list_objects(Bucket=bucket_name, Prefix=prefix, Delimiter='/')
            if not any((objects.get('CommonPrefixes'),
                        (objects.get('Contents')))):
                exit('Nothing found in: {0}'.format(item[:-1]))
            common_prefixes = objects.get('CommonPrefixes', list())
            folders = list()
            for first_bit in common_prefixes:
                folders.append(first_bit)
            output_s3_list(objects=objects, folders=folders, item=item, bucket_name=bucket_name)
        except ClientError as ce:
            if 'NoSuchBucket' in ce.response['Error']['Code']:
                exit('Bucket not found.')
            else:
                exit('Unhandled error: {0}'.format(ce.response['Error']['Code']))


@handle_boto_errors
def s3_info(aws_config=None, item=None):
    """
    @type aws_config: Config
    @type item: unicode
    """
    s3_client = get_client(client_type='s3', config=aws_config)
    prefix = ''
    if item and '/' in item:
        path_elements = item.split('/')
        bucket_name = path_elements[0]
        prefix = "/".join(path_elements[1:])
        if prefix.endswith('/'):
            prefix = prefix[:-1]
    else:
        bucket_name = item
    buckets = s3_client.list_buckets()
    owner = buckets.get('Owner')

    try:
        if bucket_name:
            check_bucket_accessible(s3_client=s3_client, bucket_name=bucket_name)
            s3_object = s3_client.get_object(Bucket=bucket_name, Key=prefix)
            output_s3_info(s3_object=s3_object, key=prefix, bucket=bucket_name)
        else:
            output_s3_info(owner=owner)
    except ClientError as ce:
        if 'NoSuchBucket' in ce.response['Error']['Code']:
            exit('Bucket not found.')
        elif 'NoSuchKey' in ce.response['Error']['Code']:
            exit('Key not found.')
        else:
            exit('Unhandled error: {0}'.format(ce.response['Error']['Code']))


@handle_boto_errors
def s3_cp(aws_config=None, source=None, dest=None):
    """
    @type aws_config: Config
    @type source: unicode
    @type dest: unicode
    """
    from acli.utils import (is_readable)
    from boto3.s3.transfer import S3Transfer, TransferConfig
    import os
    config = TransferConfig(
        multipart_threshold=200 * 1024 * 1024,
        max_concurrency=10,
        num_download_attempts=10,
    )
    s3_prefix = 's3://'
    s3_client = get_client(client_type='s3', config=aws_config)
    if source.startswith(s3_prefix) and not dest.startswith(s3_prefix):
        # COPYING FROM S3 TO LOCAL
        s3_location = source[5:].split('/')
        bucket_name = s3_location[0]
        s3_source = '/'.join(s3_location[1:])
        check_bucket_accessible(s3_client=s3_client, bucket_name=bucket_name)
        if dest == '/':
            dest = '{0}{1}'.format(os.path.abspath(dest), s3_location[-1])
        elif dest == '.' or dest.endswith('/'):
            dest = '{0}/{1}'.format(os.path.abspath(dest), s3_location[-1])
        elif os.path.isdir(os.path.abspath(dest)):
            dest = '{0}/{1}'.format(dest, s3_location[-1])
        s3_file_md5 = get_s3_file_md5(s3_client=s3_client,
                                      bucket_name=bucket_name,
                                      path=s3_source)
        if s3_file_md5:
            if s3_file_md5 == get_local_file_md5(dest):
                exit('Not transferring as identical file already exists.')
        else:
            exit('Cannot find: {0}/{1}'.format(bucket_name, s3_source))
        transfer = S3Transfer(s3_client, config)
        try:
            print('Transferring: {0} to: {1}'.format(source, dest))
            transfer.download_file(bucket_name, s3_source, dest)
        except BaseException as e:
            if hasattr(e, 'strerror') and e.strerror == 'Permission denied':
                exit('Permission denied.')
            else:
                print('Unhandled exception: {0}'.format(e))
    elif source.startswith(s3_prefix) and dest.startswith(s3_prefix):
        # COPYING FROM S3 TO S3
        print('Transferring: {0} to: {1}'.format(source, dest))
        exit('Not yet implemented.')
    elif not source.startswith(s3_prefix) and dest.startswith(s3_prefix):
        try:
            # COPYING ITEM(S) FROM LOCAL TO S3
            if os.path.isdir(source):
                exit('File transfers only for now.')
            else:
                # COPY LOCAL FILE TO S3
                if not is_readable(source):
                    exit('Cannot access: {0}'.format(source))
                s3_location = dest[5:].split('/')
                bucket_name = s3_location[0]
                s3_dest = '/'.join(s3_location[1:])
                # COPYING FILE TO A FOLDER
                if dest.endswith('/'):
                    file_name = source.split('/')[-1]
                    s3_dest += file_name
                check_bucket_accessible(s3_client=s3_client, bucket_name=bucket_name)
                # CHECK IF FILES ARE IDENTICAL
                s3_file_md5 = get_s3_file_md5(s3_client=s3_client,
                                              bucket_name=bucket_name,
                                              path=s3_dest)
                # If it's mulipart, then don't bother checking and just transfer
                if s3_file_md5 and '-' not in s3_file_md5:
                    local_file_md5 = get_local_file_md5(path=source)
                    if local_file_md5 == s3_file_md5:
                        exit('Not transferring as identical file already exists.')
                print('Transferring: {0} to: {1}'.format(source, dest))
                transfer = S3Transfer(s3_client, config)
                transfer.upload_file(source, bucket_name, s3_dest)
        except ClientError as ce:
            if 'AccessDenied' in ce.response['Error']['Code']:
                exit('Access denied. Please check permissions.')
        except Exception as e:
            print('Unhandled exception: {0}'.format(e))
    else:
        exit('Source or dest must be an S3 location defined with s3://.')
    exit()


@handle_boto_errors
def s3_rm(aws_config=None, item=None):
    """
    @type aws_config: Config
    @type item: unicode
    """
    s3_client = get_client(client_type='s3', config=aws_config)
    prefix = ''
    bucket_name = ''
    if item and '/' in item:
        path_elements = item.split('/')
        bucket_name = path_elements[0]
        prefix = "/".join(path_elements[1:])
        if prefix.endswith('/'):
            exit('Only keys can currently be removed.')
        check_bucket_accessible(s3_client=s3_client, bucket_name=bucket_name)
    else:
        exit('Invalid key.')
    try:
        s3_client.head_object(Bucket=bucket_name, Key=prefix)
    except ClientError:
        exit('Cannot access \'{0}\'.'.format(item))
    try:
        s3_client.delete_objects(Bucket=bucket_name, Delete={'Objects': [{'Key': prefix}, ]})
        exit('\'{0}\' deleted.'.format(item))
    except ClientError as error:
        if 'NoSuchBucket' in error.response['Error']['Code']:
            exit('Bucket not found.')
        elif 'NoSuchKey' in error.response['Error']['Code']:
            exit('Key not found.')
        else:
            exit('Unhandled error: {0}'.format(error.response['Error']['Code']))



# -*- coding: utf-8 -*-
"""Usage:
    acli clean (delete_orphaned_snapshots | delete_unnamed_volumes) [options]

    -n, --noop      Don't run but show what would be changed.
    -h, --help
"""
from __future__ import (absolute_import, print_function, unicode_literals)
from docopt import docopt

if __name__ == '__main__':
    print(docopt(__doc__))



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, print_function, unicode_literals)

import re

from colorclass import Color, Windows

from acli.output import (output_ascii_table, output_ascii_table_list, dash_if_none)

Windows.enable(auto_colors=True, reset_atexit=True)


def output_s3_list(buckets=None, bucket_name=None,
                   objects=None, folders=None, item=None):
    """
    @type buckets: dict
    @type bucket_name: unicode
    @type objects: dict
    @type bucket_name: unicode
    @type objects: dict
    @type folders: list
    @type item: unicode
    """
    to_remove_len = 0
    if bucket_name:
        to_remove_len = len(re.sub(bucket_name, '', item)) - 1
    if buckets:
        sorted_buckets = sorted(buckets, key=lambda k: ['Name'])
        td = list()
        td.append([Color('{autoblue}name{/autoblue}'), Color('{autoblue}created{/autoblue}')])
        for bucket in sorted_buckets:
            td.append([bucket.get('Name'),
                       str(bucket.get('CreationDate').replace(tzinfo=None, microsecond=0))])
        output_ascii_table(table_title=Color('{autowhite}s3 buckets{/autowhite}'),
                           table_data=td,
                           inner_heading_row_border=True)

    if any((objects, folders)):
        td = list()
        table_header = [Color('{autoblue}item{/autoblue}'), Color('{autoblue}size (bytes){/autoblue}'),
                        Color('{autoblue}last modified (UTC){/autoblue}'),
                        Color('{autoblue}class{/autoblue}'), Color('{autoblue}etag{/autoblue}')]
        if folders:
            for folder in folders:
                td.append([Color('{autogreen}' + folder.get('Prefix')[to_remove_len:] + '{/autogreen}'),
                           Color('{autoblack}-{/autoblack}'),
                           Color('{autoblack}-{/autoblack}'),
                           Color('{autoblack}-{/autoblack}'),
                           Color('{autoblack}-{/autoblack}')])
        object_list = objects.get('Contents')
        if object_list:
            sorted_object_list = sorted(object_list, key=lambda k: ['Key'])
            for an_object in sorted_object_list:
                an_object_key = an_object.get('Key')[to_remove_len:]
                if an_object_key:
                    td.append([an_object_key,
                               str(an_object.get('Size')),
                               str(an_object.get('LastModified').replace(tzinfo=None, microsecond=0)),
                               str(an_object.get('StorageClass')),
                               str(an_object.get('ETag')[1:-1])])
            if not folders and not td:
                td.append(['', ' ', ' ', ' ', ' '])
        output_ascii_table_list(table_title=Color('{autowhite}' + item + '{/autowhite}'),
                                table_data=td,
                                table_header=table_header,
                                inner_heading_row_border=True)
    exit(0)


def output_s3_info(s3_object=None, key=None, bucket=None, owner=None):
    """
    @type s3_object: dict
    @type key: unicode
    @type bucket: unicode
    @type owner: dict
    """
    if key:
        td = list()
        last_modified = str(s3_object['LastModified'].replace(tzinfo=None, second=0))
        etag = str(s3_object['ETag'])[1:-1]
        multipart = 'yes' if '-' in etag else 'no'
        td.append([Color('{autoblue}name{/autoblue}'),
                   str(key.split('/')[-1])])
        td.append([Color('{autoblue}last modified (UTC){/autoblue}'),
                   last_modified])
        td.append([Color('{autoblue}ETag{/autoblue}'),
                   etag])
        td.append([Color('{autoblue}size (bytes){/autoblue}'),
                   str(s3_object['ContentLength'])])
        td.append([Color('{autoblue}content type{/autoblue}'),
                   str(s3_object['ContentType'])])
        td.append([Color('{autoblue}multipart{/autoblue}'),
                   multipart])
        td.append([Color('{autoblue}version id{/autoblue}'),
                   str(dash_if_none(s3_object.get('VersionId')))])
        table_title = '{0}/{1}/'.format(bucket, '/'.join(key.split('/')[:-1]))
        output_ascii_table(table_title=Color('{autowhite}' + table_title + '{/autowhite}'),
                           table_data=td)
    else:
        td = list()
        td.append([Color('{autoblue}Owner{/autoblue}'), Color('{autoblue}Owner ID{/autoblue}')])
        td.append([owner.get('DisplayName'), owner.get('ID')])
        output_ascii_table(table_title=Color('{autowhite}s3 buckets owner{/autowhite}'),
                           table_data=td,
                           inner_heading_row_border=True)
    exit(0)



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, print_function, unicode_literals)

from colorclass import Color, Windows

from acli.output import (output_ascii_table_list, dash_if_none)

Windows.enable(auto_colors=True, reset_atexit=True)


def colour_state(state=None):
    if not state:
        return Color('{autoblack}-{/autoblack}')
    elif state == 'running':
        return Color('{autogreen}' + state + '{/autogreen}')
    elif state in ('stopped', 'stopping', 'shutting-down', 'terminated'):
        return Color('{autored}' + state + '{/autored}')
    elif state in ('rebooting', 'pending'):
        return Color('{autoyellow}' + state + '{/autoyellow}')


def output_snapshot_list(snapshots=None):
    """
    @type snapshots: list
    """
    td = list()
    table_header = [Color('{autoblue}name{/autoblue}'),
                    Color('{autoblue}id{/autoblue}'),
                    Color('{autoblue}size (GiB){/autoblue}'),
                    Color('{autoblue}description{/autoblue}'),
                    Color('{autoblue}status{/autoblue}'),
                    Color('{autoblue}started{/autoblue}'),
                    Color('{autoblue}progress{/autoblue}'),
                    Color('{autoblue}encrypted{/autoblue}')]
    for snapshot in snapshots:
        td.append([dash_if_none(snapshot.get('Tags')),
                   dash_if_none(snapshot.get('SnapshotId')),
                   dash_if_none(snapshot.get('VolumeSize')),
                   dash_if_none(snapshot.get('Description')),
                   dash_if_none(snapshot.get('State')),
                   dash_if_none(snapshot.get('StartTime (UTC)')),
                   dash_if_none(snapshot.get('Progress')),
                   str(snapshot.get('Encrypted'))])
    output_ascii_table_list(table_title=Color('{autowhite}orphaned snapshots{/autowhite}'),
                            table_data=td,
                            table_header=table_header,
                            inner_heading_row_border=True)
    exit(0)



# -*- coding: utf-8 -*-
"""Usage:
    acli asg (ls | list) [options]
    acli asg  info <asg_name> [options]
    acli asg (cpu | mem | net | delete) <asg_name> [options]

    -s, --start=<start_date>        metrics start-date
    -e, --end=<end_date>            metrics end-date
    -p, --period<period>            metrics period
    -i, --intervals=<intervals>     metrics intervals
    -o, --output=<output_type>      table, json, yaml or graph [default: graph].
    -h, --help
"""
from __future__ import (absolute_import, print_function, unicode_literals)
from docopt import docopt
from acli.services import (cloudwatch, asg)


def asg_command(argv=None, aws_config=None):
    asg_res = docopt(__doc__, argv=argv)
    if any((asg_res.get('ls'), asg_res.get('list'))):
        asg.asg_list(aws_config)
    elif asg_res.get('info'):
        asg.asg_info(aws_config, asg_name=asg_res.get('<asg_name>'))
    elif asg_res.get('cpu'):
        cloudwatch.asg_cpu(aws_config=aws_config,
                           asg_name=asg_res.get('<asg_name>'),
                           output_type=asg_res.get('--output'),
                           start=asg_res.get('--start'),
                           period=asg_res.get('--end'),
                           intervals=asg_res.get('intervals')
                           )
    elif asg_res.get('delete'):
        asg.asg_delete(aws_config, asg_name=asg_res.get('<asg_name>'))


if __name__ == '__main__':
    print(docopt(__doc__))



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, print_function, unicode_literals)
import sys
import os
import platform
import base64


BASH_COMPLETION_SCRIPT = "TBC"
BASH_COMPLETION_PATH_OSX = "/usr/local/etc/bash_completion.d"
BASH_COMPLETION_PATH_LINUX = "/etc/bash_completion.d"


def install_completion():
    if platform.system() == 'Darwin':
        if os.path.exists(BASH_COMPLETION_PATH_OSX):
            with open("{0}/acli".format(BASH_COMPLETION_PATH_OSX), 'w') as acli_file:
                acli_file.write(base64.b64decode(BASH_COMPLETION_SCRIPT))
            sys.exit("bash completion script written to: {0}/acli\nrestart terminal to use.".format(
                BASH_COMPLETION_PATH_OSX)
            )
        else:
            sys.exit("bash completion not installed. try 'brew install bash-completion'.")
    elif platform.system() == 'Linux':
        if os.path.exists(BASH_COMPLETION_PATH_LINUX):
            with open("{0}/acli".format(BASH_COMPLETION_PATH_LINUX), 'w') as acli_file:
                acli_file.write(base64.b64decode(BASH_COMPLETION_SCRIPT))
            sys.exit("bash completion script written to: {0}/acli\nrestart terminal to use.".format(
                BASH_COMPLETION_PATH_LINUX)
            )
        else:
            sys.exit("bash completion not installed.")

    else:
        sys.exit("Shell completion only available on Linux and OS X.")


def get_console_dimensions():
    try:
        rows, columns = os.popen('stty size', 'r').read().split()
    except ValueError:
        rows, columns = 80, 25
    return rows, columns


def is_readable(path=None):
    """Test if the supplied filesystem path can be read
    :param path: A filesystem path
    :return: True if the path is a file that can be read. Otherwise, False.
    """
    if os.path.isfile(path) and os.access(path, os.R_OK):
        return True
    return False


def is_writeable(path=None):
    """Test if the supplied filesystem path can be written to
    :param path: A filesystem path
    :return: True if the path is a file that can be written. Otherwise, False.
    """
    if os.path.isfile(path) and os.access(path, os.W_OK):
        return True


def get_tag_value(name=None, tags=None):
    if tags:
        for tag in tags:
            if tag.get('Key') == name:
                return tag.get('Value')



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, print_function, unicode_literals)
try:
    import configparser
except ImportError:
    from external.six.moves import configparser
import os


class Config(object):
    def __init__(self, cli_args):
        self.access_key_id = None
        self.secret_access_key = None
        self.region = None
        self.config_loader(cli_args)

    def is_config_loaded(self):
        if self.access_key_id and self.secret_access_key and self.region:
            return True

    def config_loader(self, cli_args):
        """ Load configuration from sources in order of precedence:
        CLI, ENV, BOTO
        """
        self.from_cli(cli_args)
        self.load_acli_config()

    def from_cli(self, cli_args):
        cli_aws_region = cli_args.get('--region')
        cli_access_key_id = cli_args.get('--access_key_id')
        cli_secret_access_key = cli_args.get('--secret_access_key')
        if cli_aws_region:
            self.region = cli_aws_region
        if cli_access_key_id:
            self.access_key_id = cli_access_key_id
        if cli_secret_access_key:
            self.secret_access_key = cli_secret_access_key

    @staticmethod
    def load_acli_config():
        if os.path.isfile('acli.cfg'):
            configparser.read('acli.cfg')



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, print_function, unicode_literals)

from acli.connections import get_client
from acli.errors import handle_boto_errors
from acli.output.iam import (output_iam_user_list, output_iam_summary, output_iam_user_info)
from botocore.exceptions import ClientError


# from acli.output.iam import (output_iam_user_list, output_iam_user_info)


@handle_boto_errors
def iam_user_list(aws_config=None):
    """
    @type aws_config: Config
    """
    iam_client = get_client(client_type='iam', config=aws_config)
    users_response = iam_client.list_users()
    mfa_devices_response = iam_client.list_virtual_mfa_devices(AssignmentStatus='Assigned')
    mfa_devices = mfa_devices_response.get('VirtualMFADevices')
    users = users_response.get('Users')
    if users:
        output_iam_user_list(users=users, mfa_devices=mfa_devices)
    exit('No users found.')


def iam_user_info(aws_config=None, username=None):
    """
    @type aws_config: Config
    @type username: str
    """
    iam_client = get_client(client_type='iam', config=aws_config)
    try:
        user_response = iam_client.get_user(UserName=username)
        user = user_response['User']

        user_access_keys_response = iam_client.list_access_keys(UserName=username)
        user_access_keys = user_access_keys_response.get('AccessKeyMetadata')

        user_policies_response = iam_client.list_user_policies(UserName=username)
        user_policies = user_policies_response.get('PolicyNames')

        user_groups_response = iam_client.list_groups_for_user(UserName=username)
        user_groups = user_groups_response.get('Groups')
        group_names = list()
        for ug in user_groups:
            group_names.append(ug['GroupName'])

        mfa_devices_response = iam_client.list_virtual_mfa_devices(AssignmentStatus='Assigned')
        mfa_devices = mfa_devices_response.get('VirtualMFADevices')
        user_mfa_devices = list()
        for mfa_device in mfa_devices:
            if mfa_device['User']['UserName'] == username:
                user_mfa_devices.append(mfa_device)
        output_iam_user_info(user=user, user_mfa_devices=user_mfa_devices, user_access_keys=user_access_keys,
                             user_policies=user_policies, user_groups=group_names)
    except (ClientError, IndexError):
        exit("Cannot find user: {0}".format(username))


def summary(aws_config=None):
    """
    @type aws_config: Config
    """
    iam_client = get_client(client_type='iam', config=aws_config)
    summary_response = iam_client.get_account_summary()
    summary_map = summary_response['SummaryMap']
    output_iam_summary(summary_map=summary_map)

#
# @handle_boto_errors
# def eip_info(aws_config=None, eip=None):
#     """
#     @type aws_config: Config
#     @type eip: unicode
#     """
#     ec2_client = get_client(client_type='ec2', config=aws_config)
#     try:
#         addresses_response = ec2_client.describe_addresses(PublicIps=[eip])
#         address = addresses_response.get('Addresses')[0]
#         if address:
#             output_eip_info(address=address)
#     except ClientError:
#         exit('EIP {0} not found.'.format(eip))



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, print_function, unicode_literals)

from botocore.exceptions import ClientError

from acli.connections import get_client
from acli.errors import handle_boto_errors
from acli.output.efs import output_filesystems, output_filesystem_info


@handle_boto_errors
def efs_list(aws_config=None):
    """
    @type aws_config: Config
    """
    efs_client = get_client(client_type='efs', config=aws_config)
    res = efs_client.describe_file_systems()
    filesystems = res.get('FileSystems')
    mount_targets = list()
    for fs in filesystems:
        res = efs_client.describe_mount_targets(FileSystemId=fs.get('FileSystemId'))
        for mt in res.get('MountTargets'):
            mount_targets.append(mt)
    if filesystems:
        output_filesystems(filesystems=filesystems, mount_targets=mount_targets)
    else:
        exit("No file systems found.")


@handle_boto_errors
def efs_info(aws_config=None, filesystem_id=None):
    """
    @type aws_config: Config
    @type filesystem_id: unicode
    """
    efs_client = get_client(client_type='efs', config=aws_config)
    try:
        filesystem = efs_client.describe_file_systems(FileSystemId=filesystem_id)
        mount_targets = list()
        res = efs_client.describe_mount_targets(FileSystemId=filesystem['FileSystems'][0]['FileSystemId'])
        for mt in res.get('MountTargets'):
            mount_targets.append(mt)
        output_filesystem_info(filesystem=filesystem, mount_targets=mount_targets)
    except (ClientError, IndexError):
        exit("Cannot find filesystem: {0}".format(filesystem_id))



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, print_function, unicode_literals)
import datetime
from acli.output.cloudwatch import (output_ec2_cpu, output_ec2_net,
                                    output_asg_cpu, output_ec2_vols)
from acli.services.ec2 import ec2_get_instance_vols
from acli.connections import get_client
from acli.errors import handle_boto_errors


@handle_boto_errors
def ec2_net(aws_config=None, instance_id=None, intervals=None, period=None,
            start=None, end=datetime.datetime.utcnow()):
    """
    @type aws_config: Config
    @type instance_id: unicode
    @type intervals: int
    @type period: int
    @type start: datetime
    @type end: datetime
    """
    if not intervals:
        intervals = 60
    if not period:
        period = 7200
    cloudwatch_client = get_client(client_type='cloudwatch', config=aws_config)
    if not start:
        start = end - datetime.timedelta(seconds=period)
    net_in = cloudwatch_client.get_metric_statistics(
            Namespace='AWS/EC2',
            MetricName='NetworkIn',
            Dimensions=[{'Name': 'InstanceId', 'Value': instance_id}],
            StartTime=start,
            EndTime=datetime.datetime.utcnow(),
            Period=intervals,
            Statistics=['Average'],
            Unit='Bytes'
        )
    net_out = cloudwatch_client.get_metric_statistics(
            Namespace='AWS/EC2',
            MetricName='NetworkOut',
            Dimensions=[{'Name': 'InstanceId', 'Value': instance_id}],
            StartTime=start,
            EndTime=datetime.datetime.utcnow(),
            Period=intervals,
            Statistics=['Average'],
            Unit='Bytes'
        )

    net_in_datapoints, net_out_datapoints = net_in.get('Datapoints'), net_out.get('Datapoints')
    if not all((net_in_datapoints, net_out_datapoints)):
        exit("Metrics unavailable.")
    sorted_net_in_datapoints = sorted(net_in_datapoints, key=lambda v: v.get('Timestamp'))
    sorted_net_out_datapoints = sorted(net_out_datapoints, key=lambda v: v.get('Timestamp'))
    in_dates = [x1.get('Timestamp') for x1 in sorted_net_in_datapoints]
    in_values = [x2.get('Average') for x2 in sorted_net_in_datapoints]
    out_dates = [x3.get('Timestamp') for x3 in sorted_net_out_datapoints]
    out_values = [x4.get('Average') for x4 in sorted_net_out_datapoints]
    output_ec2_net(in_dates=in_dates, in_values=in_values,
                   out_dates=out_dates, out_values=out_values,
                   instance_id=instance_id)
    exit(0)


@handle_boto_errors
def ec2_cpu(aws_config=None, instance_id=None, intervals=None, period=None,
            start=None, end=datetime.datetime.utcnow()):
    """
    @type aws_config: Config
    @type instance_id: unicode
    @type intervals: int
    @type period: int
    @type start: datetime
    @type end: datetime
    """
    if not intervals:
        intervals = 60
    if not period:
        period = 7200
    cloudwatch_client = get_client(client_type='cloudwatch', config=aws_config)
    if not start:
        start = end - datetime.timedelta(seconds=period)
    out = cloudwatch_client.get_metric_statistics(
            Namespace='AWS/EC2',
            MetricName='CPUUtilization',
            Dimensions=[
                {
                 'Name': 'InstanceId',
                 'Value': instance_id
                }
            ],
            StartTime=start,
            EndTime=datetime.datetime.utcnow(),
            Period=intervals,
            Statistics=[
                'Average',
            ],
            Unit='Percent'
        )
    datapoints = out.get('Datapoints')
    sorted_datapoints = sorted(datapoints, key=lambda v: v.get('Timestamp'))
    dates = list()
    values = list()
    for datapoint in sorted_datapoints:
        dates.append(datapoint.get('Timestamp'))
        values.append(datapoint.get('Average'))
    output_ec2_cpu(dates=dates, values=values, instance_id=instance_id)
    exit(0)


@handle_boto_errors
def asg_cpu(aws_config=None, asg_name=None, start=None, period=None, intervals=None,
            output_type=None):
    """
    @type aws_config: Config
    @type asg_name: unicode
    @type intervals: int
    @type period: int
    @type start: datetime
    @type output_type: unicode
    """
    end = datetime.datetime.utcnow()
    if not output_type or output_type == 'graph':
        if not intervals:
            intervals = 60
        if not period:
            period = 7200
        cloudwatch_client = get_client(client_type='cloudwatch', config=aws_config)
        if not start:
            start = end - datetime.timedelta(seconds=period)
        out = cloudwatch_client.get_metric_statistics(
                Namespace='AWS/EC2',
                MetricName='CPUUtilization',
                Dimensions=[
                    {
                     'Name': 'AutoScalingGroupName',
                     'Value': asg_name
                    }
                ],
                StartTime=start,
                EndTime=datetime.datetime.utcnow(),
                Period=intervals,
                Statistics=[
                    'Average',
                ],
                Unit='Percent'
            )
        datapoints = out.get('Datapoints')
        sorted_datapoints = sorted(datapoints, key=lambda v: v.get('Timestamp'))
        dates = [y1.get('Timestamp') for y1 in sorted_datapoints]
        values = [y2.get('Average') for y2 in sorted_datapoints]

        output_asg_cpu(dates=dates, values=values, asg_name=asg_name)
        exit(0)
    elif output_type == 'table':
        print("table")
        exit(0)


@handle_boto_errors
def ec2_vol(aws_config=None, instance_id=None, intervals=None, period=None,
            start=None, end=datetime.datetime.utcnow()):
    """
    @type aws_config: Config
    @type instance_id: unicode
    @type intervals: int
    @type period: int
    @type start: datetime
    @type end: datetime
    """
    ebs_vols = ec2_get_instance_vols(aws_config=aws_config, instance_id=instance_id)
    if not intervals:
        intervals = 60
    if not period:
        period = 7200

    cloudwatch_client = get_client(client_type='cloudwatch', config=aws_config)
    if not start:
        start = end - datetime.timedelta(seconds=period)
    vol_datapoints = list()
    for ebs_vol in ebs_vols:
        read_ops = cloudwatch_client.get_metric_statistics(
                Namespace='AWS/EBS',
                MetricName='VolumeReadBytes',
                Dimensions=[{'Name': 'VolumeId', 'Value': ebs_vol['Ebs']['VolumeId']}],
                StartTime=start,
                EndTime=datetime.datetime.utcnow(),
                Period=intervals,
                Statistics=['Average'],
                Unit='Bytes'
        )
        write_ops = cloudwatch_client.get_metric_statistics(
                Namespace='AWS/EBS',
                MetricName='VolumeWriteBytes',
                Dimensions=[{'Name': 'VolumeId', 'Value': ebs_vol['Ebs']['VolumeId']}],
                StartTime=start,
                EndTime=datetime.datetime.utcnow(),
                Period=intervals,
                Statistics=['Average'],
                Unit='Bytes'
            )
        sorted_read_datapoints = sorted(read_ops.get('Datapoints'), key=lambda v: v.get('Timestamp'))
        sorted_write_datapoints = sorted(write_ops.get('Datapoints'), key=lambda v: v.get('Timestamp'))
        read_dates = [x1.get('Timestamp') for x1 in sorted_read_datapoints]
        read_values = [x2.get('Average') for x2 in sorted_read_datapoints]
        write_dates = [y1.get('Timestamp') for y1 in sorted_write_datapoints]
        write_values = [y2.get('Average') for y2 in sorted_write_datapoints]
        vol_datapoints.append({'device_name': ebs_vol['DeviceName'],
                               'read_dates': read_dates, 'read_values': read_values,
                               'write_dates': write_dates, 'write_values': write_values})
    output_ec2_vols(vols_datapoints=vol_datapoints, instance_id=instance_id)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
usage: acli [--version] [--help] [--install-completion]
       acli [--region=<region>] [--access_key_id=<access_key_id>] [--secret_access_key=<secret_access_key>]
            <command> [<args>...]
       acli account [options]
       acli ec2 (ls | list | summary) [options]
       acli ec2 (start | stop | reboot | terminate | info) <instance_id> [options]
       acli lc (ls | list) [options]
       acli asg (ls | list) [options]
       acli asg (info) <asg_name> [options]
       acli ami (ls | list) info <ami_id>
       acli ami info <ami_id>
       acli eip ([ls | list]| info <eip>)
       acli eip (ls | list)
       acli elb (ls | list)
       acli elb info <elb_name>
       acli route53 (ls | list)
       acli route53 info <zone_id>
       acli vpc (ls | list)
       acli vpc info <vpc_id>
       acli secgroup (ls | list)
       acli secgroup info <secgroup_id>
       acli s3 (ls | list) [<item>]
       acli s3 info [<item>]
       acli s3 cp <source> <dest>
       acli s3 (del | rm) <item>
       acli es (ls | list)
       acli es info <domain>
       acli efs (ls | list) [options]
       acli efs (info) <filesystem_id> [options]
       acli iam summary [options]
       acli iam user (ls | list) [options]
       acli iam user info <username>
       acli clean (delete_orphaned_snapshots | delete_unnammed_volumes)


options:
   -h, --help      help

The most common commands are:
   account      Get account info
   ec2          Manage ec2 instances
   eip          Manage elastic IPs
   elb          Manage elb instances
   ami          Manage amis
   asg          Manage auto-scaling groups
   lc           Manage launch configurations
   iam          Manage iam
   eip          Manage elastic ips
   secgroup     Manage security groups
   route53      Manage route 53 configuration
   vpc          Manage VPCs
   secgroup     Manage Security Groups
   s3           Manage S3 storage
   es           Manage Elasticsearch
   efs          Manage Elastic file systems
   clean        Tools for reporting on and removing unused service items
See 'acli <command> -h'
"""

from __future__ import (absolute_import,
                        print_function,
                        unicode_literals)

from botocore.exceptions import ClientError
from docopt import docopt

from acli.commands.asg import asg_command
from acli.commands.ec2 import ec2_command
from acli.config import Config
from acli.services import (ec2, elb, account, vpc, eip, asg,
                           route53, secgroup, s3, es, clean, efs, iam)
from acli.utils import install_completion


def elb_command(argv=None, aws_config=None):
    from acli.commands import elb as command_elb
    elb_res = docopt(command_elb.__doc__, argv=argv)
    if any((elb_res.get('ls'), elb_res.get('list'))):
        elb.elb_list(aws_config)
    elif elb_res.get('info'):
        elb.elb_info(aws_config, elb_name=elb_res.get('<elb_name>'))


def ami_command(argv=None, aws_config=None):
    from acli.commands import ami as command_ami
    ami_res = docopt(command_ami.__doc__, argv=argv)
    if any((ami_res.get('ls'), ami_res.get('list'))):
        ec2.ami_list(aws_config, filter_term=ami_res.get('--filter'))
    elif ami_res.get('info'):
        ec2.ami_info(aws_config, ami_id=ami_res.get('<ami_id>'))


def lc_command(argv=None, aws_config=None):
    from acli.commands import lc as command_lc
    lc_res = docopt(command_lc.__doc__, argv=argv)
    if any((lc_res.get('ls'), lc_res.get('list'))):
        asg.lc_list(aws_config)
    elif lc_res.get('info'):
        asg.lc_info(aws_config, lc_name=lc_res.get('<lc_name>'))


def iam_command(argv=None, aws_config=None):
    from acli.commands import iam as command_iam
    iam_res = docopt(command_iam.__doc__, argv=argv)
    if iam_res.get('user'):
        if any((iam_res.get('ls'), iam_res.get('list'))):
            iam.iam_user_list(aws_config)
        if iam_res.get('info'):
            iam.iam_user_info(aws_config, username=iam_res.get('<username>'))
    if iam_res.get('summary'):
        iam.summary(aws_config)


def route53_command(argv=None, aws_config=None):
    from acli.commands import route53 as command_route53
    route53_res = docopt(command_route53.__doc__, argv=argv)
    if any((route53_res.get('ls'), route53_res.get('list'))):
        route53.route53_list(aws_config)
    elif route53_res.get('info'):
        route53.route53_info(aws_config, zone_id=route53_res.get('<zone_id>'))


def vpc_command(argv=None, aws_config=None):
    from acli.commands import vpc as command_vpc
    vpc_res = docopt(command_vpc.__doc__, argv=argv)
    if any((vpc_res.get('ls'), vpc_res.get('list'))):
        vpc.vpc_list(aws_config)
    elif vpc_res.get('info'):
        vpc.vpc_info(aws_config, vpc_id=vpc_res.get('<vpc_id>'))


def secgroup_command(argv=None, aws_config=None):
    from acli.commands import secgroup as command_secgroup
    secgroup_res = docopt(command_secgroup.__doc__, argv=argv)
    if any((secgroup_res.get('ls'), secgroup_res.get('list'))):
        secgroup.secgroup_list(aws_config)
    elif secgroup_res.get('info'):
        secgroup.secgroup_info(aws_config, secgroup_id=secgroup_res.get('<secgroup_id>'))


def eip_command(argv=None, aws_config=None):
    from acli.commands import eip as command_eip
    eip_res = docopt(command_eip.__doc__, argv=argv)
    if any((eip_res.get('ls'), eip_res.get('list'))):
        eip.eip_list(aws_config)
    elif eip_res.get('info'):
        eip.eip_info(aws_config, eip=eip_res.get('<eip>'))


def s3_command(argv=None, aws_config=None):
    from acli.commands import s3 as command_s3
    s3_res = docopt(command_s3.__doc__, argv=argv)
    if any((s3_res.get('ls'), s3_res.get('list'))):
        path = s3_res.get('<item>')
        if path and not '/' == path[-1:]:
            path += '/'
        s3.s3_list(aws_config, item=path)
    elif s3_res.get('info'):
        s3.s3_info(aws_config, item=s3_res.get('<item>'))
    elif s3_res.get('cp'):
        s3.s3_cp(aws_config, source=s3_res.get('<source>'), dest=s3_res.get('<dest>'))
    elif any((s3_res.get('rm'), s3_res.get('del'))):
        s3.s3_rm(aws_config, item=s3_res.get('<item>'))


def es_command(argv=None, aws_config=None):
    from acli.commands import es as command_es
    es_res = docopt(command_es.__doc__, argv=argv)
    if any((es_res.get('ls'), es_res.get('list'))):
        es.es_list(aws_config)
    elif es_res.get('info'):
        es.es_info(aws_config, domain_name=es_res.get('<domain>'))


def efs_command(argv=None, aws_config=None):
    from acli.commands import efs as command_efs
    efs_res = docopt(command_efs.__doc__, argv=argv)
    if any((efs_res.get('ls'), efs_res.get('list'))):
        efs.efs_list(aws_config)
    elif efs_res.get('info'):
        efs.efs_info(aws_config, filesystem_id=efs_res.get('<filesystem_id>'))


def clean_command(argv=None, aws_config=None):
    from acli.commands import clean as command_clean
    clean_res = docopt(command_clean.__doc__, argv=argv)
    if clean_res.get('delete_orphaned_snapshots'):
        clean.delete_orphaned_snapshots(aws_config=aws_config, noop=clean_res.get('--noop'))
    elif clean_res.get('delete_unnamed_volumes'):
        clean.delete_unnamed_volumes(aws_config=aws_config, noop=clean_res.get('--noop'))


def real_main():
    args = docopt(__doc__,
                  version='0.1.33',
                  options_first=True)
    aws_config = Config(args)
    argv = [args['<command>']] + args['<args>']
    try:
        if args['--install-completion']:
            install_completion()
        elif args['<command>'] == 'account':
            account.account_info(aws_config)
        elif args['<command>'] == 'ec2':
            ec2_command(argv=argv, aws_config=aws_config)
        elif args['<command>'] == 'elb':
            elb_command(argv=argv, aws_config=aws_config)
        elif args['<command>'] == 'lc':
            lc_command(argv=argv, aws_config=aws_config)
        elif args['<command>'] == 'iam':
            iam_command(argv=argv, aws_config=aws_config)
        elif args['<command>'] == 'asg':
            asg_command(argv=argv, aws_config=aws_config)
        elif args['<command>'] == 'ami':
            ami_command(argv=argv, aws_config=aws_config)
        elif args['<command>'] == 'route53':
            route53_command(argv=argv, aws_config=aws_config)
        elif args['<command>'] == 'vpc':
            vpc_command(argv=argv, aws_config=aws_config)
        elif args['<command>'] == 'secgroup':
            secgroup_command(argv=argv, aws_config=aws_config)
        elif args['<command>'] == 'eip':
            eip_command(argv=argv, aws_config=aws_config)
        elif args['<command>'] == 's3':
            s3_command(argv=argv, aws_config=aws_config)
        elif args['<command>'] == 'es':
            es_command(argv=argv, aws_config=aws_config)
        elif args['<command>'] == 'efs':
            efs_command(argv=argv, aws_config=aws_config)
        elif args['<command>'] == 'clean':
            clean_command(argv=argv, aws_config=aws_config)
        elif args['<command>'] in ['help'] and not args['<args>']:
            print("usage: acli <command> -h")
        elif args['<command>']:
            exit("{0} is not an acli command. See 'acli -h'.".format(args['<command>']))
        else:
            print(__doc__)
    except ClientError as ce:
        error_code = ce.response['Error']['Code']
        if error_code == 'InvalidClientTokenId':
            exit('The access key ID is invalid (AWS Code: InvalidClientTokenId).')
        if error_code == 'AccessDenied':
            exit('You are not authorized to perform this action (AWS Code: AccessDenied).')

        if error_code == 'AuthFailure':
            exit('AWS was unable to validate the provided credentials (AWS Code: AuthFailure).')

        if error_code == 'InvalidToken':
            exit('Provided token is invalid (AWS Code: InvalidToken).')
        else:
            raise



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, print_function, unicode_literals)

from colorclass import Color, Windows

from acli.output import (output_ascii_table, output_ascii_table_list, dash_if_none)

Windows.enable(auto_colors=True, reset_atexit=True)


def colour_staus(status=None):
    if not status:
        return Color('{autoblack}-{/autoblack}')
    elif status == 'Active':
        return Color('{autogreen}' + status + '{/autogreen}')
    elif status == 'Inactive':
        return Color('{autored}' + status + '{/autored}')


def user_has_mfa_assigned(username=None, password_last_used=None, mfa_devices=None):
    for mfa_device in mfa_devices:
        if mfa_device['User']['UserName'] == username:
            return Color('{autogreen}TRUE{/autogreen}')
    if password_last_used:
        return Color('{autored}FALSE{/autored}')
    else:
        return Color('FALSE')


def output_iam_user_list(users=None, mfa_devices=None):
    """
    @type users: list
    @type mfa_devices: list
    """
    td = list()
    table_header = [Color('{autoblue}user name{/autoblue}'),
                    Color('{autoblue}created{/autoblue}'),
                    Color('{autoblue}password last used{/autoblue}'),
                    Color('{autoblue}mfa enabled{/autoblue}')]
    for user in users:
        td.append([dash_if_none(user.get('UserName')),
                   dash_if_none(user.get('CreateDate')),
                   dash_if_none(user.get('PasswordLastUsed')),
                   user_has_mfa_assigned(username=user.get('UserName'), password_last_used=user.get('PasswordLastUsed'),
                                         mfa_devices=mfa_devices)])
    output_ascii_table_list(table_title=Color('{autowhite}user list{/autowhite}'),
                            table_data=td,
                            table_header=table_header,
                            inner_heading_row_border=True)
    exit(0)


def output_iam_summary(summary_map=None):
    """
    @type summary_map: dict
    """
    td = list()
    td.append([Color('{autoblue}users{/autoblue}'),
               dash_if_none(summary_map.get('Users'))])
    td.append([Color('{autoblue}groups{/autoblue}'),
               dash_if_none(summary_map.get('Groups'))])
    td.append([Color('{autoblue}roles{/autoblue}'),
               dash_if_none(summary_map.get('Roles'))])
    td.append([Color('{autoblue}policies{/autoblue}'),
               summary_map.get('Policies')])
    td.append([Color('{autoblue}account access keys present{/autoblue}'),
               summary_map.get('AccountAccessKeysPresent')])
    td.append([Color('{autoblue}policy versions in use{/autoblue}'),
               summary_map.get('PolicyVersionsInUse')])
    td.append([Color('{autoblue}server certificates{/autoblue}'),
               summary_map.get('ServerCertificates')])
    td.append([Color('{autoblue}mfa devices{/autoblue}'),
               dash_if_none(summary_map.get('MFADevices'))])
    td.append([Color('{autoblue}mfa devices in use{/autoblue}'),
               summary_map.get('MFADevicesInUse')])
    td.append([Color('{autoblue}account MFA enabled{/autoblue}'),
               dash_if_none(summary_map.get('AccountMFAEnabled'))])
    td.append([Color('{autoblue}providers{/autoblue}'),
               summary_map.get('Providers')])
    td.append([Color('{autoblue}instance profiles{/autoblue}'),
               dash_if_none(summary_map.get('InstanceProfiles'))])
    td.append(["", ""])
    td.append([Color('{autowhite}quotas{/autowhite}'), ""])

    td.append([Color('{autoblue}user policy size quota{/autoblue}'),
               dash_if_none(summary_map.get('UserPolicySizeQuota'))])
    td.append([Color('{autoblue}assume role policy size quota{/autoblue}'),
               dash_if_none(summary_map.get('AssumeRolePolicySizeQuota'))])
    td.append([Color('{autoblue}server certificates quota{/autoblue}'),
               dash_if_none(summary_map.get('ServerCertificatesQuota'))])
    td.append([Color('{autoblue}users quota{/autoblue}'),
               dash_if_none(summary_map.get('UsersQuota'))])

    td.append([Color('{autoblue}policy size quota{/autoblue}'),
               dash_if_none(summary_map.get('PolicySizeQuota'))])

    td.append([Color('{autoblue}attached policies per group quota{/autoblue}'),
               dash_if_none(summary_map.get('AttachedPoliciesPerGroupQuota'))])

    td.append([Color('{autoblue}groups per user quota{/autoblue}'),
               dash_if_none(summary_map.get('GroupsPerUserQuota'))])
    td.append([Color('{autoblue}groups quota{/autoblue}'),
               dash_if_none(summary_map.get('GroupsQuota'))])
    td.append([Color('{autoblue}instance profiles quota{/autoblue}'),
               dash_if_none(summary_map.get('InstanceProfilesQuota'))])
    td.append([Color('{autoblue}attached policies per role quota{/autoblue}'),
               dash_if_none(summary_map.get('AttachedPoliciesPerRoleQuota'))])
    td.append([Color('{autoblue}access keys per user quota{/autoblue}'),
               dash_if_none(summary_map.get('AccessKeysPerUserQuota'))])
    td.append([Color('{autoblue}group policy size quota{/autoblue}'),
               dash_if_none(summary_map.get('GroupPolicySizeQuota'))])
    td.append([Color('{autoblue}versions per policy quota{/autoblue}'),
               dash_if_none(summary_map.get('VersionsPerPolicyQuota'))])

    td.append([Color('{autoblue}roles quota{/autoblue}'),
               dash_if_none(summary_map.get('RolesQuota'))])

    td.append([Color('{autoblue}policy version in use quota{/autoblue}'),
               dash_if_none(summary_map.get('PolicyVersionsInUseQuota'))])
    td.append([Color('{autoblue}policies quota{/autoblue}'),
               dash_if_none(summary_map.get('PoliciesQuota'))])
    td.append([Color('{autoblue}role policy size quota{/autoblue}'),
               dash_if_none(summary_map.get('RolePolicySizeQuota'))])
    output_ascii_table(table_title=Color('{autowhite}iam account summary{/autowhite}'),
                       table_data=td)
    exit(0)


def output_iam_user_info(user=None, user_mfa_devices=None, user_access_keys=None, user_policies=None, user_groups=None):
    """
    @type user: dict
    @type user_mfa_devices: list
    @type user_access_keys: list
    @type user_policies: list
    @type user_groups: list
    """
    td = list()
    td.append([Color('{autoblue}name{/autoblue}'),
               dash_if_none(user.get('UserName'))])
    td.append([Color('{autoblue}id{/autoblue}'),
               dash_if_none(user.get('UserId'))])
    td.append([Color('{autoblue}arn{/autoblue}'),
               dash_if_none(user.get('Arn'))])
    td.append([Color('{autoblue}path{/autoblue}'),
               dash_if_none(user.get('Path'))])
    td.append([Color('{autoblue}password last used{/autoblue}'),
               dash_if_none(user.get('PasswordLastUsed'))])
    td.append([Color('{autoblue}created{/autoblue}'),
               dash_if_none(user.get('CreateDate'))])
    if user_groups:
        td.append([Color('{autoblue}groups{/autoblue}'), ",".join(user_groups)])
    else:
        td.append([Color('{autoblue}groups{/autoblue}'),
                   dash_if_none()])
    td.append([Color('{autoblue}policies{/autoblue}'),
               dash_if_none(",".join(user_policies))])
    if not user_access_keys:
        td.append([Color('{autoblue}access keys{/autoblue}'),
                   dash_if_none()])
    else:
        td.append([Color('{autowhite}access keys{/autowhite}'), ""])
        for key in user_access_keys:
            td.append([Color('{autoblue}  id / status / enabled{/autoblue}'),
                       "{0} / {1} / {2}".format(key.get('AccessKeyId'), colour_staus(key.get('Status')),
                                                key.get('CreateDate'))])
    if not user_mfa_devices:
        td.append([Color('{autoblue}mfa devices{/autoblue}'),
                   dash_if_none()])
    else:
        td.append([Color('{autowhite}mfa devices{/autowhite}'), ""])
        for md in user_mfa_devices:
            td.append([Color('{autoblue}  serial / enabled{/autoblue}'),
                       "{0} / {1}".format(md.get('SerialNumber'), md.get('EnableDate'))])

    output_ascii_table(table_title=Color('{autowhite}user info{/autowhite}'),
                       table_data=td)
    exit(0)



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, print_function, unicode_literals)

import botocore.exceptions

from acli.connections import get_client
from acli.errors import handle_boto_errors
from acli.output.elb import output_elbs, output_elb_info


@handle_boto_errors
def get_elb_list(aws_config):
    """
    @type aws_config: Config
    """
    elb_client = get_client(client_type='elb', config=aws_config)
    elbs = elb_client.describe_load_balancers().get('LoadBalancerDescriptions')
    return elbs


@handle_boto_errors
def get_elb(aws_config, elb_name=None):
    """
    @type aws_config: Config
    @type elb_name: unicode
    """
    if elb_name:
        try:
            elb_client = get_client(client_type='elb', config=aws_config)
            elbs = elb_client.describe_load_balancers(LoadBalancerNames=[elb_name])
            if elbs and elbs.get('LoadBalancerDescriptions'):
                return elbs.get('LoadBalancerDescriptions')[0]
        except botocore.exceptions.ClientError:
            exit('ELB: {0} could not be found.'.format(elb_name))


def elb_list(aws_config):
    """
    @type aws_config: Config
    """
    output_elbs(elbs=get_elb_list(aws_config))


def elb_info(aws_config=None, elb_name=None):
    """
    @type aws_config: Config
    @type elb_name: unicode
    """
    output_elb_info(elb=get_elb(aws_config,
                                elb_name=elb_name))



#!/usr/bin/env python

import os
import sys
from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand
sys.path.insert(0, os.path.abspath('lib'))

version = "0.1.34"

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload -r pypi')
    # os.system('python setup.py bdist_wheel upload')
    sys.exit()

if sys.argv[-1] == 'tag':
    os.system("git tag -a {0} -m 'version {1}'".format(version, version))
    os.system("git push --follow-tags")
    sys.exit()

readme = open('README.md').read()
history = open('HISTORY.rst').read().replace('.. :changelog:', '')

long_description = readme + '\n\n' + history

if sys.argv[-1] == 'readme':
    print(long_description)
    sys.exit()


class PyTest(TestCommand):
    user_options = [('pytest-args=', 'a', "Arguments to pass to py.test")]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.pytest_args = []

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        import pytest
        errno = pytest.main(self.pytest_args)
        sys.exit(errno)

setup(
    name='acli',
    version=version,
    author='Jon Hadfield',
    author_email='jon@lessknown.co.uk',
    url='http://github.com/jonhadfield/acli',
    download_url='https://github.com/jonhadfield/acli/tarball/{0}'.format(version),
    install_requires=['docopt>=0.6.2',
                      'colorclass>=2.2.0',
                      'botocore>=1.4.39',
                      'boto3>=1.3.1',
                      'requests',
                      'terminaltables>=3.0.0',
                      'humanize>=0.5.1'],
    description='A CLI to manage AWS resources',
    long_description=long_description,
    packages=find_packages('lib'),
    platforms='any',
    license='MIT',
    scripts=['bin/acli'],
    package_dir={'': 'lib'},
    include_package_data=True,
    classifiers=[
        'Programming Language :: Python',
        'Development Status :: 3 - Alpha',
        'Natural Language :: English',
        'Environment :: Console',
        'Intended Audience :: System Administrators',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Topic :: System :: Networking',
    ],
    keywords=(
        'aws, Python, cli, amazon, ec2'
    ),
    tests_require=['pytest-cov>=2.2.0',
                   'pytest>=2.8.2',
                   'moto>=0.4.24',
                   'matplotlib>=1.5.1'],
    cmdclass={'test': PyTest},
)



from __future__ import (absolute_import, print_function, unicode_literals)
from acli.services.vpc import (vpc_list, vpc_info)
from acli.config import Config
from moto import mock_ec2
from acli.connections import get_client

import pytest
from boto3.session import Session
session = Session(region_name="eu-west-1")


config = Config(cli_args={'--region': 'eu-west-1',
                          '--access_key_id': 'AKIAIOSFODNN7EXAMPLE',
                          '--secret_access_key': 'wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMPLEKEY'})


@pytest.yield_fixture(scope='function')
def fake_vpcs():
    """VPC mock service"""
    mock = mock_ec2()
    mock.start()
    ec2_client = get_client(client_type='ec2', config=config)
    vpc = ec2_client.create_vpc(CidrBlock='10.0.0.0/16')
    print(vpc)
    ec2_client.create_subnet(VpcId=vpc['Vpc']['VpcId'], CidrBlock="10.0.0.0/18")
    yield ec2_client.describe_vpcs()
    mock.stop()


@pytest.yield_fixture(scope='function')
def fake_empty_vpcs():
    """VPC mock service"""
    mock = mock_ec2()
    mock.start()
    yield None
    mock.stop()


def test_vpc_list_service(fake_vpcs):
    with pytest.raises(SystemExit):
        assert vpc_list(aws_config=config)


def test_vpc_list_service_empty(fake_empty_vpcs, capsys):
    with pytest.raises(SystemExit):
        out, err = capsys.readouterr(vpc_list(aws_config=config))
        assert err == "No VPCs found."


def test_vpc_info_service(fake_vpcs):
    with pytest.raises(SystemExit):
        vpc_info(aws_config=config, vpc_id=fake_vpcs.get('Vpcs')[0].get('VpcId'))
        assert vpc_info(aws_config=config, vpc_id=fake_vpcs.get('Vpcs')[0].get('VpcId'))


def test_vpc_info_service_empty(fake_empty_vpcs, capsys):
    with pytest.raises(SystemExit):
        invalid_vpc_id = 'invalid'
        out, err = capsys.readouterr(vpc_info(aws_config=config, vpc_id=invalid_vpc_id))
        assert err == "Cannot find VPC: {0}".format(invalid_vpc_id)



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, print_function, unicode_literals)

from acli.connections import get_client
from acli.errors import handle_boto_errors
from acli.output.snapshots import (output_snapshot_list)


@handle_boto_errors
def delete_orphaned_snapshots(aws_config=None, noop=False):
    """
    @type aws_config: Config
    @type noop: bool
    """
    account_id = None
    try:
        iam_client = get_client(client_type='iam', config=aws_config)
        users = iam_client.list_users(MaxItems=1)
        if users.get('Users'):
            account_id = users.get('Users')[0]['Arn'].split(':')[4]
    except:
        exit('Unable to get account details. Please check your permissions.')
    if account_id:
        ec2_client = get_client(client_type='ec2', config=aws_config)
        desc_snapshots_result = ec2_client.describe_snapshots(OwnerIds=[account_id])
        snapshots = desc_snapshots_result.get('Snapshots')
        desc_images_result = ec2_client.describe_images()
        images = desc_images_result['Images']
        image_id_list = [image['ImageId'] for image in images]

        orphaned_snapshots = list()
        for snapshot in snapshots:
            snapshot_description = snapshot.get('Description')
            if snapshot_description.startswith('Created by CreateImage'):
                ami_string_start = snapshot_description.find('ami-')
                ami_string_end = snapshot_description.find(' ', ami_string_start)
                ami_id = snapshot_description[ami_string_start:ami_string_end]
                if ami_id not in image_id_list:
                    orphaned_snapshots.append(snapshot)

        if orphaned_snapshots:
            if not noop:
                print("Total snapshots: {}".format(len(snapshots)))
                print("Orphaned snapshots: {}".format(len(orphaned_snapshots)))
                total_deleted = 0
                try:
                    for index, orphaned_snapshot in enumerate(orphaned_snapshots, start=1):
                        ec2_client.delete_snapshot(SnapshotId=orphaned_snapshot.get('SnapshotId'))
                        total_deleted = index

                except:
                    print('An error occurred whilst deleting snapshots.')
                exit('Deleted {0} out of a possible {1}.'.format(total_deleted, len(orphaned_snapshots)))

            else:
                output_snapshot_list(snapshots=orphaned_snapshots)
        else:
            print('No orphaned snapshots were found.')


def delete_unnamed_volumes(aws_config=None, noop=False):
    """
    @type aws_config: Config
    @type noop: bool
    """
    ec2_client = get_client(client_type='ec2', config=aws_config)
    desc_volumes_result = ec2_client.describe_volumes()
    volumes = desc_volumes_result.get('Volumes')
    total_volumes = len(volumes)
    volumes_to_delete = list()
    for volume in volumes:
        volume_has_name = False
        volume_tags = volume.get('Tags')
        if volume_tags:
            for volume_tag in volume_tags:
                if volume_tag.get('Key') and volume_tag.get('Key') == 'Name':
                    volume_has_name = True
                    break
        if not volume_has_name and not volume.get('Attachments'):
            volumes_to_delete.append(volume)
    total_volumes_to_delete = len(volumes_to_delete)
    total_volumes_deleted = 0
    if not noop:
        try:
            for index, volume_to_delete in enumerate(volumes_to_delete, start=1):
                ec2_client.delete_volume(VolumeId=volume_to_delete.get('VolumeId'))
                total_volumes_deleted = index
        except:
            print('An error occurred whislt deleting volumes.')
        if total_volumes_to_delete:
            exit('Deleted {0} volumes out of a total of {1} volumes '
                 'that are unnamed and unattached.'.format(total_volumes_deleted,
                                                           total_volumes_to_delete))
        else:
            exit('No unnamed and unattached volumes were found.')
    elif total_volumes_to_delete:
        exit('There are {0} volumes out of a total of {1} volumes '
             'that are unnamed and unattached.'.format(total_volumes_to_delete,
                                                       total_volumes))
    else:
        exit('No unnamed and unattached volumes were found.')



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, print_function, unicode_literals)

from boto3.session import Session


def get_boto3_session(aws_config):
    """
    @type aws_config: Config
    """
    if all((aws_config.region, aws_config.access_key_id, aws_config.secret_access_key)):
        return Session(region_name=aws_config.region,
                       aws_access_key_id=aws_config.access_key_id,
                       aws_secret_access_key=aws_config.secret_access_key)
    elif aws_config.region:
        return Session(region_name=aws_config.region)
    else:
        return Session()


def get_client(client_type=None, config=None):
    """
    @type client_type: basestring
    @type config: Config
    """
    session = get_boto3_session(aws_config=config)
    if client_type == 'ec2':
        return session.client('ec2')
    elif client_type == 'elb':
        return session.client('elb')
    elif client_type == 'iam':
        return session.client('iam')
    elif client_type == 'autoscaling':
        return session.client('autoscaling')
    elif client_type == 'cloudwatch':
        return session.client('cloudwatch')
    elif client_type == 'route53':
        return session.client('route53')
    elif client_type == 's3':
        return session.client('s3')
    elif client_type == 'es':
        return session.client('es')
    elif client_type == 'efs':
        return session.client('efs')



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, print_function, unicode_literals)

from botocore.exceptions import ClientError

from acli.connections import get_client
from acli.errors import handle_boto_errors
from acli.output.es import (output_domain_list, output_domain_info)


@handle_boto_errors
def es_list(aws_config=None):
    """
    @type aws_config: Config
    """
    es_client = get_client(client_type='es', config=aws_config)
    domains = es_client.list_domain_names()
    domain_names = domains.get('DomainNames')
    if domain_names:
        output_domain_list(domains=domain_names)
    else:
        exit("No domains found.")


@handle_boto_errors
def es_info(aws_config=None, domain_name=None):
    """
    @type aws_config: Config
    @type domain_name: unicode
    """
    es_client = get_client(client_type='es', config=aws_config)
    try:
        domain = es_client.describe_elasticsearch_domains(DomainNames=[domain_name])
        output_domain_info(domain=domain)
    except (ClientError, IndexError):
        exit("Cannot find domain: {0}".format(domain_name))
